<?php

namespace Drupal\Tests\acquia_contenthub_subscriber\Kernel\EventSubscriber\EntityDataTamper;

use Acquia\ContentHubClient\CDFDocument;
use Drupal\acquia_contenthub\Event\EntityDataTamperEvent;
use Drupal\acquia_contenthub_subscriber\EventSubscriber\EntityDataTamper\SkipUserAndUserRoleSyndication;
use Drupal\acquia_contenthub_subscriber\Libs\UserSyndicationSettingsInterface;
use Drupal\depcalc\DependencyStack;
use Drupal\KernelTests\Core\Entity\EntityKernelTestBase;
use Drupal\Tests\acquia_contenthub\Kernel\Traits\CdfDocumentCreatorTrait;
use Drupal\user\UserInterface;

/**
 * Tests user entity tampers.
 *
 * @coversDefaultClass \Drupal\acquia_contenthub_subscriber\EventSubscriber\EntityDataTamper\SkipUserAndUserRoleSyndication
 *
 * @requires module depcalc
 *
 * @group acquia_contenthub_subscriber
 */
class SkipUserSyndicationTest extends EntityKernelTestBase {

  use CdfDocumentCreatorTrait;

  protected const ORIGINAL_USER_UUID = '995f955b-08a9-4436-a0c7-1cde093ee174';

  /**
   * User settings.
   *
   * @var \Drupal\acquia_contenthub_subscriber\Libs\UserSyndicationSettingsInterface
   */
  protected UserSyndicationSettingsInterface $userSettings;

  /**
   * SUT.
   *
   * @var \Drupal\acquia_contenthub_subscriber\EventSubscriber\EntityDataTamper\SkipUserAndUserRoleSyndication
   */
  protected SkipUserAndUserRoleSyndication $sut;

  /**
   * Proxy user.
   *
   * @var \Drupal\user\UserInterface
   */
  protected UserInterface $proxyUser;

  /**
   * Cdf document.
   *
   * @var \Acquia\ContentHubClient\CDFDocument
   */
  protected CDFDocument $cdf;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'acquia_contenthub_subscriber',
    'acquia_contenthub',
    'depcalc',
  ];

  /**
   * {@inheritDoc}
   */
  public function setUp(): void {
    parent::setUp();
    $this->userSettings = $this->container->get('acquia_contenthub_subscriber.user_syndication.settings');
    $this->proxyUser = $this->createUser();
    $this->sut = new SkipUserAndUserRoleSyndication($this->userSettings, $this->container->get('entity_type.manager'));
    $this->cdf = $this->createCdfDocumentFromFixtureFile('node/node_page.json');
  }

  /**
   * Tests that original user is replaced with proxy user.
   */
  public function testCdfTamperWithProxyUser(): void {
    $this->userSettings->toggleUserSyndication(TRUE);
    $this->userSettings->setProxyUser($this->proxyUser->id());
    $stack = $this->tamperCdf();
    $original_user = $this->cdf->getCdfEntity(self::ORIGINAL_USER_UUID);
    $this->assertTrue($stack->hasDependency($original_user->getUuid()), 'Original user is added to the stack as processed entity.');
    $this->assertEquals($this->proxyUser->uuid(), $stack->getDependency($original_user->getUuid())->getUuid(), 'Original user was replaced with proxy user.');
  }

  /**
   * Tests that existing user is added to stack.
   */
  public function testCdfTamperWithExistingUser(): void {
    $this->userSettings->toggleUserSyndication(TRUE);
    // This should have no effect
    // when original user already exists on subscriber.
    $this->userSettings->setProxyUser($this->proxyUser->id());
    $email = 'admin@example.com';
    // Create original user.
    $this->createUser([
      'mail' => $email,
    ]);
    $stack = $this->tamperCdf();
    $original_user = $this->cdf->getCdfEntity(self::ORIGINAL_USER_UUID);
    $this->assertTrue($stack->hasDependency($original_user->getUuid()), 'Original user is added to the stack as processed entity.');
    /** @var \Drupal\user\UserInterface $existing_user */
    $existing_user = $stack->getDependency($original_user->getUuid())->getEntity();
    $this->assertEquals($email, $existing_user->getEmail(), 'Existing user was loaded with same email address.');
  }

  /**
   * Tests that nothing happens if user syndication is disabled.
   *
   * And no proxy user is set.
   */
  public function testCdfTamperWithNoProxyUser(): void {
    $this->userSettings->toggleUserSyndication(TRUE);
    $stack = $this->tamperCdf();
    $original_user = $this->cdf->getCdfEntity(self::ORIGINAL_USER_UUID);
    $this->assertFalse($stack->hasDependency($original_user->getUuid()));
    $this->assertFalse($stack->hasDependency($this->proxyUser->uuid()));
    $this->assertEmpty($stack->getDependencies());
  }

  /**
   * Tests that nothing happens if user syndication is enabled.
   *
   * Even if proxy user is set.
   *
   * @throws \Exception
   */
  public function testCdfTamperWithUserSyndicationEnabled(): void {
    $this->userSettings->setProxyUser($this->proxyUser->id());
    $stack = $this->tamperCdf();
    $original_user = $this->cdf->getCdfEntity(self::ORIGINAL_USER_UUID);
    $this->assertFalse($stack->hasDependency($original_user->getUuid()));
    $this->assertFalse($stack->hasDependency($this->proxyUser->uuid()));
    $this->assertEmpty($stack->getDependencies());
  }

  /**
   * Tampers cdf document with data tamper event subscriber.
   *
   * @return \Drupal\depcalc\DependencyStack
   *   Dependency stack.
   *
   * @throws \Exception
   */
  protected function tamperCdf(): DependencyStack {
    $event = new EntityDataTamperEvent($this->cdf, new DependencyStack());
    $this->sut->onDataTamper($event);
    return $event->getStack();
  }

}
